import java.util.*;

import iBus.*;
import iBus.layers.*;
import iBus.exception.*;

import Ticker.*;

/* ***************************************************************************
 * Classe: Consumer (persistente)
 *
 */
public class Consumer {
  static String appName = "Consumer";
  static Log log = new Log(appName);			// Objecto log
  static String channelVName = "Tick-Channel";
  static String channelVVer = "1.5";
  static String channelVSubject = "/P";
  static String channelVQoS = "Reliable";

  static String channelAName = channelVName;
  static String channelAVer = channelVVer;
  static String channelASubject = "/A";
  static String channelAQoS = "Reliable";

  public static void main(String argv[]) throws Exception {
    Application.setAppName(appName);

    /* ***********************************************************************
     * Instancia a stack para o canal volatil (V).
     *
     * Os membros do canal volatil sao os Produtores (talkers),
     * os Consumidores (listeners) e os membros do Arquivo (listeners).
     */

    // Pilha de protocolos para difusao fiavel (reliable multicast)
    final iBus.Stack stackV = new iBus.Stack(channelVQoS);

    // Aumenta os valores do timeout e heartbeat para tornar a deteccao de
    // falhas menos agressiva (valores em milisegundos). Por outro lado,
    // assumimos que este canal nao perde mensagens.
    REACH reachV = (REACH)stackV.getProtocolObject("REACH");
    reachV.setTimeout(60000);
    reachV.setHbeat(10000);
    NAK nakV = (NAK)stackV.getProtocolObject("NAK");
    nakV.setHBInterval(60000);

    iBusURL urlV = null;				// Endereco do canal V
    try {
      urlV = iBusURLFactory.create(channelVName, channelVVer, channelVSubject);
    }
    catch (Exception e) {
      // Coloca no log uma mensagem e termina a aplicacao (exit code=1)
      log.panic("main: iBusURLFactory.create failed for channel P: " + e);
    }


    /* ***********************************************************************
     * Define a aplicacao como um consumidor do canal V.
     *
     * A aplicacao participa nos canais A e V como consumidor. As mensagens
     * nao recebidas em V sao depois solicitadas ao Arquivo e recebidas em A.
     */

    // Instancia o objecto que recebe as mensagens enviadas em V
    ObjectReceiver rcvV = new ObjectReceiver(log);

    // Conecta-se ao canal V como assinante (provoca uma mudanca de vista em V)
    try {
      stackV.subscribe(urlV, rcvV);
    }
    catch (Exception e) {
      log.panic("main: stack.subscribe failed for channel P: " + e);
    }


    /* ***********************************************************************
     * Instancia a stack para o canal do aquivo (A).
     *
     * Este canal e utilizado para receber as mensagens nao recebidas em V.
     * Desta forma o Consumidor pode ser sincronizado com os restantes
     * consumidores do canal V.
     */

    final iBus.Stack stackA = new iBus.Stack(channelAQoS);

    // Aumenta os valores do timeout e heartbeat para tornar a deteccao de
    // falhas menos agressiva (valores em milisegundos). Tal como em V, tambem
    // assumimos que o canal A nao perde mensagens.
    REACH reachA = (REACH)stackA.getProtocolObject("REACH");
    reachA.setTimeout(60000);
    reachA.setHbeat(10000);
    NAK nakA = (NAK)stackA.getProtocolObject("NAK");
    nakA.setHBInterval(60000);

    iBusURL urlA = null;				// Endereco do canal A
    try {
      urlA = iBusURLFactory.create(channelAName, channelAVer, channelASubject);
    }
    catch (Exception e) {
      // Coloca no log uma mensagem e termina a aplicacao (exit code=1)
      log.panic("main: iBusURLFactory.create failed for channel A: " + e);
    }


    /* ***********************************************************************
     * Define a aplicacao como um consumidor do canal A.
     *
     */

    // Instancia o objecto que recebe as mensagens enviadas em A
    ObjectReceiver rcvA = new ObjectReceiver(log);

    // Conecta-se ao canal A como assinante (provoca uma mudanca de vista em A)
    try {
      stackA.subscribe(urlA, rcvA);
    }
    catch (Exception e) {
      log.panic("main: stack.subscribe failed for channel A: " + e);
    }


    /* ***********************************************************************
     * Inicio da aplicacao
     *
     */

    System.out.println(Application.getAppName() + ": inicio da aplicacao.\n");

    stackV.waitTillExit();			// Aguarda por mensagens em V
  }
}



/* ***************************************************************************
 * Classe: ObjectReceiver
 *
 */
class ObjectReceiver implements iBus.Receiver {
  Log log;						// Objecto log
  static int count = 0;

  // Constructor
  public ObjectReceiver(Log log_) {
    log = log;
  }

  // O metodo dispatchPush e' invocado quando uma mensagem e' recebida
  public void dispatchPush(iBusURL channel, Posting posting) {
    for (int n = 0; n < posting.getLength(); n++) {
      String postingName = posting.getObject(n).getClass().getName();

      if (postingName.substring(0, 7).equals("Ticker.")) {
        Tick post = (Tick)posting.getObject(n);
        String sender = post.getSenderId();
        int order = post.getOrderNumber();

        System.out.println((count++) + "\t" + new Date().toString());
      }
      else {
        String str = (String)posting.getObject(n);
        System.out.println("Mensagem desconhecida: " + str.toString());
      }
    }

    System.out.flush();
  }

  // O metodo dispatchPull e' invocado quando uma mensagem e' enviada
  public Posting dispatchPull(iBusURL channel, Posting request) {
    System.err.println("ObjectReceiver.dispatchPull called");
    return(null);
  }

  // O metodo error e' invocado quando ocorre um erro
  public void error(iBusURL channel, String details) {
    log.panic("ObjectReceiver: internal error: " + details);
  }
}

// fim: Consumer.java
